@@ -18,7 +18,7 @@ from .fileWatch import *
 class SelfNamed(object):
 	"""docstring for SelfNamed."""
 	__id = "xxx"
-	languageDefaults = ["value","_val","getattr","show","_id","__dict__"]
+	_languageDefaults = ["value","_val","getattr","show","_id","__dict__"]
 	def __init__(self, id = None):
 		super().__init__()
 		pass #print("|||||||||||||||||||||||||||||||||||||||||||||||||||")
@@ -89,7 +89,7 @@ class Expando(SelfNamed):
 
 	def tree(self):
 		for a in self.__dict__:
-			if not a.startswith("_") and a not in SelfNamed.languageDefaults:
+			if not a.startswith("_") and a not in SelfNamed._languageDefaults:
 				yield self[a]
 				if self[a] != None:
 					for z in self[a].tree():
@@ -97,7 +97,7 @@ class Expando(SelfNamed):
 
 	def subscribe(self, func = None, autoPub = None, block = False, once= False, echo=True, debug = False, withID = False):
 		if func is None:
-			func = lambda a : print("xObject changed:",a)
+			func = lambda a, *aa, **aaa : [a,aa,aaa]
 			withID = True
 		channel = self._id.replace(".","/")
 		# print("CCCCCCCCCCCCCCCCCC",channel)
@@ -119,7 +119,7 @@ class Expando(SelfNamed):
 	def kids(self):
 
 				for a in self.__dict__:
-					if not a.startswith("_") and a not in SelfNamed.languageDefaults:
+					if not a.startswith("_") and a not in SelfNamed._languageDefaults:
 						yield self[a]
 
 
@@ -128,7 +128,7 @@ class Expando(SelfNamed):
 		# childs = []
 		childs = {}
 		for a in self.__dict__:
-			if not a.startswith("_") and a not in SelfNamed.languageDefaults:
+			if not a.startswith("_") and a not in SelfNamed._languageDefaults:
 				# childs.append(self[a])
 				childs[a]=self[a]
 		return childs
@@ -197,7 +197,7 @@ class Expando(SelfNamed):
 		pass ## print("EEEEEEEEEEEEEEEEEEEE1")
 		if "str" not in str(type(name)):
 			name = str(name)
-		if not name.startswith("_") and "_val" in self.__dict__ and name not in SelfNamed.languageDefaults:#### and "__skip" in self.__dict__ and name not in self.skip:
+		if not name.startswith("_") and "_val" in self.__dict__ and name not in SelfNamed._languageDefaults:#### and "__skip" in self.__dict__ and name not in self.skip:
 			if "xo.obj" not in str(type(value)):
 				pass ## print("_____________________",str(type(value)))
 				if name not in self.__dict__:
@@ -228,7 +228,7 @@ class Expando(SelfNamed):
 			# return atr[0]
 			return atr
 
-		if not name.startswith("_") and "_val" in self.__dict__ and name not in SelfNamed.languageDefaults and name not in self.__dict__:
+		if not name.startswith("_") and "_val" in self.__dict__ and name not in SelfNamed._languageDefaults and name not in self.__dict__:
 			# print("ppp44444")
 			self.__dict__[name] = obj(id = self._id+"/"+name, parent = self)
 
@@ -251,7 +251,7 @@ class Expando(SelfNamed):
 		pass #print("iiiiiiiiiiiiiiiiioooooooo")
 		if "str" not in str(type(name)):
 			name = str(name)
-		if not name.startswith("_") and "_val" in self.__dict__ and name not in SelfNamed.languageDefaults:#### and "__skip" in self.__dict__ and name not in self.skip:
+		if not name.startswith("_") and "_val" in self.__dict__ and name not in SelfNamed._languageDefaults:#### and "__skip" in self.__dict__ and name not in self.skip:
 			pass #print("VVVVVVVV",str(type(value)))
 			if "xo.obj" not in str(type(value)):
 				pass #print("_____________________",str(type(value)))
@@ -290,7 +290,7 @@ class Expando(SelfNamed):
 		if "str" not in str(type(name)):
 			name = str(name)
 		#### return name
-		if not name.startswith("_") and "_val" in self.__dict__ and name not in SelfNamed.languageDefaults and name not in self.__dict__:
+		if not name.startswith("_") and "_val" in self.__dict__ and name not in SelfNamed._languageDefaults and name not in self.__dict__:
 			pass #print("OOOOO_ooooooooooooooooooooo",name)####,self.__dict__)
 			pass #print("aaaaaaaaaaaaaa")
 			# print("ppp66666",self)
@@ -323,8 +323,14 @@ class Expando(SelfNamed):
 		#### self._val *= 3
 
 	def __repr__(self):
+		justShow = True
+		if justShow:
+			self.show(ret = False)
+			print()
 		# print("where are we ?")
 		recursiveDict = False
+		if self._val is None:
+			self._val = [[]]
 		ret = "{xobject "+str({str(self._name):self._val[0]})[1:-1]+f" ::: children({len(self.children())})"
 		childs = []
 		if self.children() is not None and len(self.children()) > 0:
@@ -385,10 +391,36 @@ class Expando(SelfNamed):
 		#### print("!!!!!!!!!")
 		#### print(type(other))
 		#### print()
+
+		#list append overrull
+		if self._val == None:
+			self._val = [[]]
+		if len(self._val) == 0:
+			self._val[0].append(other)
+			return self._val[0]
+		if str(type(other)) != str(type(self._val[0])):
+			if "list" not in str(type(other)):
+				other = [other]
+			if "list" not in str(type(self._val[0])):
+				self._val[0] = [self._val[0]]
+
+		if other is None and (self._val is None or self._val[0] is None):
+			return None
 		if self.__isObj(other):
 			return self._val[0] + other._val[0]
 		elif "str" in str(type(other)):
 			return str(self._val[0]) + other
+		if self._val[0] is None:
+			if "list" in str(type(other)):
+				return 	other
+			return []
+		if other is None:
+			other = []
+			return 	self._val + other
+		if "list" in str(type(other)):
+			if "list" in str(type(self._val[0])):
+				return self._val[0] + other
+			return 	self._val + other
 		return self._val[0] + other
 
 	def __radd__(self,other):
@@ -498,7 +530,7 @@ class Expando(SelfNamed):
 				# print(xo.GetXO(owner)+key, v,retXO=True)
 			for a in appendToLearn:
 				if self._parent is not None:
-					self._parent.learned.value(ref=True).append(a)
+					self._parent.learned+=[a]
 				else:
 					print(" ::: WTF")
 					self.value(ref=True).append(a)
@@ -551,7 +583,42 @@ class Expando(SelfNamed):
 
 
 
-	def show(self,t = "    ",count = 0):
+	def show(self,t = "    ",count = 0, inLoop = False, ret = False):
+		#### print("ssssssssssssssss..............")
+		s = ""
+		#### print("///////////",self._val,type(self._val))
+
+		if "str" in str(type(self._val)):
+			s = "\'"
+		p = self._id.split("/")[-1] +" = "+ s+str(self._val)+s
+		tab = ""
+		for i in range(count):
+			tab+=t
+
+		retList = []
+		res = []
+		p = tab+p
+		if ret:
+			retList.append(p)
+		else:
+			print(p.replace("\t","    "))
+		for a in self.__dict__:
+			# if "_" not in a:
+			if not a.startswith("_"):
+				if "xo.obj" in str(type(self.__dict__[a])):
+					if ret:
+						res = self.__dict__[a].show(count= count+1, ret = ret)
+					else:
+						self.__dict__[a].show(count= count+1, ret = ret)
+		if count is 0 and inLoop:
+			print("\n\nPress Ctrl+C to stop whileShow()\n")
+
+		if ret:
+			if count == 0:
+				return str(retList + res)
+			return retList +["\n"]+ res
+
+	def show0(self,t = "    ",count = 0, inLoop = False):
 		#### print("ssssssssssssssss..............")
 		s = ""
 		#### print("///////////",self._val,type(self._val))
@@ -570,7 +637,7 @@ class Expando(SelfNamed):
 			if not a.startswith("_"):
 				if "xo.obj" in str(type(self.__dict__[a])):
 					self.__dict__[a].show(count= count+1)
-		if count is 0:
+		if count is 0 and inLoop:
 			print("\n\nPress Ctrl+C to stop whileShow()\n")
 
 	def showMag(self,t = "    ",count = 0):
@@ -671,6 +738,7 @@ class __objManager(object):
 		#### self = data[0]
 		while True:
 
+			# print("FFFFFFF")
 			chKeys = channels.keys()
 			cKeys = []
 			discovery = False
@@ -685,12 +753,16 @@ class __objManager(object):
 						self._fin = False
 						discovery = True
 						if channels[channel]["ref"] is not None and len(channels[channel]["ref"])>1:
-							pass #print("DUPPPPPPPPPPPPPPPPPPPPPPPPPPPp")
+							# print("DUPPPPPPPPPPPPPPPPPPPPPPPPPPPp", channel)
+							pass
 						if channels[channel]["ref"] is not None and len(channels[channel]["ref"])>0:
+							# print("DUPPPPPPPPPPPPPPPPPPPPPPPPPPPpXxxx", channel)
 							channels[channel]["ref"][0][0][d] = obj(id=channel+"/"+d, parent = self.getXO(channel))
+							# print("0000000000000000000000000000000xxx")
 						else:
+							# print("DUPPPPPPPPPPPPPPPPPPPPPPPPPPPpZZZZ", channel)
 							pass #print("XRRXRXRXRXRXrXR")
-
+				# print("aaaaaaaaaaa")
 				keyC = self.loadKey(channel)
 				#### print("channel",channel)
 				try:
@@ -703,9 +775,13 @@ class __objManager(object):
 						pass
 				except:
 					pass #print("XXXXXXXX",channel)
+				# print("CCCCCCCCCCCCCCCCCCCC")
 
 			if not discovery:
 				self._fin = True
+			else:
+				pass
+				# print("DDDDDDDDDDDDDDDD")
 
 
 			time.sleep(wait)
@@ -771,8 +847,11 @@ class __objManager(object):
 			print("!!!!!!")
 			return self.zero(12)/self.zero(1)
 
+		# print("........")
 		data = self.getChannel(channel)
+		# print("........!!!!!!!")
 		self.setChannelRef(channel, ref = ref)
+		# print("........!!!!!!!!!!!!!!!!2")
 		#### if value is None:
 		#### 	value = [None]
 
@@ -784,10 +863,12 @@ class __objManager(object):
 		#### 	else:
 		#### 		self.save(channel, value)
 
+		# print("........!!!!!!!!!!!!!!!!!!!!!!!!!!!!1")
 		if value is not None:
 			#### value[0] = self.getValue(channel)
 			self.save(channel, value)
 
+		# print("........!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!0")
 		return data["_val"] #### BINDED! [var]
 
 
@@ -1001,13 +1082,16 @@ class obj(Expando):
 	def __init__(self, val = None, id = None, parent = None):
 		global manager
 		self.__id = id
+		# print("........")
 		super().__init__(val = val, id = id)
+		# print("........ddd")
 
 		self._manager = manager
 		self.__id = id
 		self._parent = parent
 		# print("!!!!!!!!", self.__id, "PPPPPPP",self._parent)
 		self._val = manager.bind(self.__id, val, ref = [self])
+		# print("........xxxxx")
 
 
 	def Del(self):
@@ -1016,22 +1100,22 @@ class obj(Expando):
 				# self._parent.__dict__[self._name] = None
 				self._parent.__dict__.pop(self._name)
 
-	def Show(self):
+	def Show(self, inLoop = False):
 		pass #print("SSSSSSSSSSSSSSSSSSSSSSSSSSSSSSS..............")
-		super().show()
+		super().show(inLoop = inLoop)
 	def ShowMag(self):
 		pass #print("SSSSSSSSSSSSSSSSSSSSSSSSSSSSSSS..............")
 		super().showMag()
 
 	def whileShow(self):
 		while(True):
-			self.show()
+			self.show(inLoop = True)
 			time.sleep(0.2)
 			os.system("clear")
 
 	def whileShowMag(self):
 		while(True):
-			self.showMag()
+			self.showMag(inLoop = True)
 			time.sleep(0.2)
 			os.system("clear")
 
@@ -1082,7 +1166,8 @@ class ok(object):
 	def _returnFull_(self, id = None, val = None):
 		pass #print("FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF")
 
-
+		timeOut = .1
+		t0 = time.time()
 		pass #print("00000000")
 		t = time.time()
 		pass #print("00000000")
@@ -1100,7 +1185,7 @@ class ok(object):
 			print()
 
 		try:
-			while(manager._fin is False):
+			while(manager._fin is False and time.time()-t0 < timeOut):
 			####
 			#### while(abs(manager._checkCounter- cycle[0])<2):
 				pass #print("waiting for ",id,"to fill",manager._checkCounter, cycle,manager._fin)
@@ -1108,7 +1193,7 @@ class ok(object):
 
 			pass #print("XXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXX",manager._checkCounter,cycle)
 		except:
-			print(f" ::: Exception while looking for xoKey {id} or setting value {val}")
+			print(f" ::: Exception while looking for xoKey {id} value {val}")
 			traceback.print_exc()
 		# print("waiting full TIME =========",time.time()-t)
 		return full
@@ -1119,17 +1204,26 @@ class ok(object):
 		#### global hasManager, Dir, ExtKey, ExtData, channels
 		# print("IIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIII")
 		global manager
-		ok._shared = self
 		# self._max = 10000000,
 		# self._wait=wait
-		print("")
-		self._manager_ = manager
 		self._Threads_ = []
 		# self._showLoading = True
 		# self._shared._Threads_ = []
 		# self._Threads_ = []
 
 		# print("_______________iiiiiiiiiiiii______________")
+		if ok._shared is not None:
+			print(" ::::::::::::::::::::::::::::::::::::::::")
+			print(" :::                                  :::")
+			print(" :::        xObject is ready          :::")
+			print(" :::                                  :::")
+			print(" :::   now you can use xo. anywhere   :::")
+			print(" :::             :)                   :::")
+			print(" :::                                  :::")
+			print(" ::::::::::::::::::::::::::::::::::::::::")
+			print()
+		self._manager_ = manager
+		ok._shared = self
 
 		#### if hasManager:
 		#### 	# print("FFFFFFFFFFFFFFFFFF")
@@ -1155,7 +1249,7 @@ class ok(object):
 	def GetXO(self, get = "", allow_creation = False, getValue = False, follow = None):
 		# print("XXXXXXXXXXXXXXXXXXXX", get, getValue)
 		if "str" not in str(type(get)):
-			print("Please provide a string as a key")
+			print(self._id,"Please provide a string as a key",get)
 			return None
 		c = 0
 		# print(c,c,c,c,c,c,c,c,c,c);c+=1
@@ -1206,6 +1300,14 @@ class ok(object):
 	## xo.wow.SetValue(1000,retXO=True).runFunc1(True, retXO=True).runFunc2("awesome") '''
 	def SetValue(self, key, value, allow_creation = True, retBool = True, retValue = False, retXO = False):
 
+		if "xo.obj" in str(type(key)):
+			if key.value() is not None:
+				key = str(key.value())
+			else:
+				key = key._id
+		if key is None:
+			" ::: SetValue Key is None :::"
+			return False
 		res = self.GetXO(key, allow_creation=allow_creation)
 		if res is not None and "xo.obj" in str(type(res)):
 			# res.show()
@@ -1286,7 +1388,7 @@ class ok(object):
 
 				pass
 			else:
-				print("File or folder does not exist",path, "doesn't exist")
+				print(" ::: File or folder does not exist",path, "doesn't exist")
 				return False
 
 
@@ -1408,7 +1510,7 @@ class ok(object):
 
 	def __get__(self,key, done = False, *args, **kwargs):
 		# print("TTTTTT",type(key))
-		print("###########",key,"#")
+		# print("###########",key,"#")
 		if done:
 			return self
 		for a in args:
@@ -1420,7 +1522,7 @@ class ok(object):
 		# for a in kwargs:
 		# 	print(a, kwargs[a])
 		# pass
-		print("eeeeeeeeeeeeeeeeeeeeeagggggggggaa")
+		# print("eeeeeeeeeeeeeeeeeeeeeagggggggggaa")
 		return [""]
 		return self.__dict__[key]
 
@@ -1437,7 +1539,7 @@ class ok(object):
 
 
 
-		if not name.startswith("_") and "_val" in self.__dict__ and name not in SelfNamed.languageDefaults:#### and "__skip" in self.__dict__ and name not in self.skip:
+		if not name.startswith("_") and "_val" in self.__dict__ and name not in SelfNamed._languageDefaults:#### and "__skip" in self.__dict__ and name not in self.skip:
 			if "xo.obj" not in str(type(value)):
 				pass ## print("_____________________",str(type(value)))
 				if name not in self.__dict__:
@@ -1465,7 +1567,7 @@ class ok(object):
 
 		return self._returnFull_(id=name)
 
-		# if not name.startswith("_") and "_val" in self.__dict__ and name not in SelfNamed.languageDefaults and name not in self.__dict__:
+		# if not name.startswith("_") and "_val" in self.__dict__ and name not in SelfNamed._languageDefaults and name not in self.__dict__:
 		# 	self.__dict__[name] = obj(id = self._id+"/"+name, parent = self)
 		#
 		# if name in self.__dict__:
@@ -1589,10 +1691,8 @@ def cleantxt(a):
 # print("XXXXXXXXXXXXXx")
 
 def changeHome(newHome):
-	print("NEED TO IMPLEMENT HOME CHANGE")
-	print("NEED TO IMPLEMENT HOME CHANGE")
-	print("NEED TO IMPLEMENT HOME CHANGE")
-	print("NEED TO IMPLEMENT HOME CHANGE", newHome)
+	pass
+	# print(" ::: NEED TO IMPLEMENT HOME CHANGE ::: ", newHome)
 
 
 dbDirName = "xo-gd"
@@ -1630,8 +1730,12 @@ loadAll = True
 if loadAll:
 	xoManager.refreshChannels()
 
-
-xo = ok()        # xo can now can generate xobjects
+if ok._shared is None:
+	# print("FFFFFFFFFFFFFFFFFFFFFf")
+	xo = ok()        # xo can now can generate xobjects
+else:
+	# print("TTTTTTTTTTTTTTTTTTTTTTTT")
+	xo = ok._shared
 # xxo = ok()        # xo can now can generate xobjects
 # xxo.main = True
 # xo = xxo.main
